--
-- BorderFlasher
--
-- Copyright (c) 2021 Dana Basken
--

BorderFlasher = {}
BorderFlasher.__index = BorderFlasher

function BorderFlasher:Create(texturePath, borderColor)
  local instance = {}
  setmetatable(instance, BorderFlasher)
  instance:Construct(texturePath, borderColor)
  return instance
end

function BorderFlasher:Construct(texturePath, borderColor)
  self.texturePath = texturePath
  self.borderColor = borderColor
  self:CreateFlasher()
end

function BorderFlasher:CreateFlasher()
  self.frame = CreateFrame("Frame", nil, UIParent, BackdropTemplateMixin and "BackdropTemplate")
  self.frame:SetFrameStrata("BACKGROUND");
  self.frame:SetClampedToScreen(true)
  self.frame:SetAllPoints(UIParent)

  local texture = self.frame:CreateTexture(nil,"BACKGROUND");
  texture:SetTexture(self.texturePath);
  texture:SetBlendMode("ADD");
  texture:SetVertexColor(self.borderColor.r, self.borderColor.g, self.borderColor.b);
  texture:SetAllPoints(self.frame);
  self.frame.texture = texture;

  self.frame:SetPoint("CENTER",0,0);
  self.frame:Hide();

  self.animationGroup = self.frame:CreateAnimationGroup()
  self.animationGroup:SetLooping("BOUNCE")

  local animation = self.animationGroup:CreateAnimation("Alpha")
  animation:SetDuration(0.25)
  animation:SetFromAlpha(0.1)
  animation:SetToAlpha(0.9)
end

function BorderFlasher:StartFlashing()
  if self.animationGroup:IsPlaying() then self:StopFlashing() end
  self.frame.texture:SetVertexColor(self.borderColor.r, self.borderColor.g, self.borderColor.b);
  self.animationGroup:Play()
  self.frame:Show()
  if self.flashTicker then self.flashTicker:Cancel() end
  self.flashTicker = C_Timer.NewTicker(1, function() self:StopFlashing() end, 0)
end

function BorderFlasher:StopFlashing()
  if self.flashTicker then self.flashTicker:Cancel() end
  self.flashTicker = nil
  self.frame:Hide()
  self.animationGroup:Stop()
end

function BorderFlasher:SetBorderColor(borderColor)
  self.borderColor = borderColor
end
