OrganizedPanicCommon = {}

function OrganizedPanicCommon:InGlobalCooldown()
  local start = GetSpellCooldown(61304)
  return start > 0
end

function OrganizedPanicCommon:InGroup()
  return OrganizedPanicCommon:InParty() or OrganizedPanicCommon:InRaid() or OrganizedPanicCommon:InLFR()
end

function OrganizedPanicCommon:InRaid()
  if not IsInGroup(LE_PARTY_CATEGORY_HOME) then return false end -- not a manual raid group
  return IsInRaid()
end

function OrganizedPanicCommon:InParty()
  if not IsInGroup(LE_PARTY_CATEGORY_HOME) then return false end -- not a manual party group
  return UnitInParty("player")
end

function  OrganizedPanicCommon:InLFR()
  if not IsInGroup(LE_PARTY_CATEGORY_INSTANCE) then return false end -- not an instance group
  return IsInRaid()
end

function OrganizedPanicCommon:GetPlayerNameWithRealm()
  return OrganizedPanicCommon:GetUnitNameWithRealm("player")
end

function OrganizedPanicCommon:GetUnitNameWithRealm(unitId)
  local name, realm = GetUnitName(unitId, true)
  if not name then return end
  if name == UNKNOWN then return end
  if realm then name = ("%s-%s"):format(name, realm) end
  return OrganizedPanicCommon:GetNameWithRealm(name)
end

function OrganizedPanicCommon:GetNameWithRealm(name)
  local name, realm = strsplit("-", name)
  if not realm then realm = GetRealmName() end
  return ("%s-%s"):format(name, realm)
end

function OrganizedPanicCommon:GetNameWithoutRealm(name)
  local name, realm = strsplit("-", name)
  return name
end

function OrganizedPanicCommon:MakeNumericVersion(version)
  local _, _, major, minor, patch, build = strfind(version, "(%d+)%.(%d+)%.(%d+)%.(%d+)")
  return (major * 1000000) + (minor * 10000) + (patch * 100) + build
end

function OrganizedPanicCommon:IsNewerVersion(version, latestVersion)
  local currentVersion = OrganizedPanicCommon:MakeNumericVersion(latestVersion)
  local comparisonVersion = OrganizedPanicCommon:MakeNumericVersion(version)
  return comparisonVersion > currentVersion
end

function OrganizedPanicCommon:GetGroupType()
  if OrganizedPanicCommon:InRaid() then return "RAID" end
  if OrganizedPanicCommon:InParty() then return "PARTY" end
  if OrganizedPanicCommon:InLFR() then return "RAID" end
end

function OrganizedPanicCommon:GetChannel()
  if OrganizedPanicCommon:InRaid() then return "RAID" end
  if OrganizedPanicCommon:InParty() then return "PARTY" end
  if OrganizedPanicCommon:InLFR() then return "INSTANCE_CHAT" end
end

function OrganizedPanicCommon:GetPlayerGroupNameForUnitId(unitId)
  local groupType = OrganizedPanicCommon:GetGroupType()
  if not groupType then return end
  local members = GetNumGroupMembers()
  for i = 1, members do
    local playerGroupName = ("%s%d"):format(groupType, i)
    if UnitGUID(playerGroupName) == unitId then return playerGroupName end
  end
end

function OrganizedPanicCommon:BooleanToString(bool)
  if bool then return "true" end
  return "false"
end

function OrganizedPanicCommon:ObjectToString(object)
  if type(object) == 'table' then
    local s = '{'
    for k, v in pairs(object) do
      s = s .. '"' .. k .. '":' .. OrganizedPanicCommon:ObjectToString(v) .. ','
    end
    return s .. '}'
  end
  if type(object) == 'string' then
    return '"' .. object .. '"'
  end
  if type(object) == "function" then
    return "fn"
  else
    return tostring(object)
  end
end

function OrganizedPanicCommon:ColorString(str, color, skipAlpha)
  if skipAlpha then return ("|c%s%s|r"):format(color, str) end
  return ("|cff%s%s|r"):format(color, str)
end

function OrganizedPanicCommon:PrefixString(prefix, str, color)
  return ("%s: %s"):format(OrganizedPanicCommon:ColorString(prefix, color), str)
end
