--
-- OrganizedPanicProgressBar
--
-- Copyright (c) 2021 Dana Basken
--

local _, NS = ...

OrganizedPanicProgressBar = {}
OrganizedPanicProgressBar.__index = OrganizedPanicProgressBar

function OrganizedPanicProgressBar:Create(parentFrame, value, max)
  local instance = {}
  setmetatable(instance, OrganizedPanicProgressBar)
  instance:Construct(parentFrame, value, max)
  return instance
end

function OrganizedPanicProgressBar:Construct(parentFrame, value, max)
  self.parentFrame = parentFrame
  self.value = value
  self.max = max
  self.text = ""
  self:CreateUI()
  self:UpdateUI()
end

function OrganizedPanicProgressBar:CreateUI()
  self:CreateContainerUI()
  self:CreateBarUI()
  self:CreateProgressUI()
  self:CreateLabelUI()
end

function OrganizedPanicProgressBar:CreateContainerUI()
  self.frame = CreateFrame("Frame", "$parentProgressBar", self.parentFrame, BackdropTemplateMixin and "BackdropTemplate")
  self.frame:SetWidth(self.parentFrame:GetWidth())
  self.frame:SetHeight(self.parentFrame:GetHeight())
  self.frame:SetPoint("CENTER", 0, 0)
  self.frame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", tile = true, tileSize = 32})
  self.frame:SetBackdropColor(0, 0, 0, 0)
end

function OrganizedPanicProgressBar:CreateBarUI()
  self.bar = CreateFrame("Frame", "$parentBar", self.frame, BackdropTemplateMixin and "BackdropTemplate")
  self.bar:SetWidth(self.frame:GetWidth())
  self.bar:SetHeight(self.frame:GetHeight() - 2)
  self.bar:SetPoint("CENTER", 0, 0)
  self.bar:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", tile = true, tileSize = 32})
  self.bar:SetBackdropColor(0, 0, 0, 0)
end

function OrganizedPanicProgressBar:CreateProgressUI()
  local background = CreateFrame("Frame", "$parentBackground", self.bar, BackdropTemplateMixin and "BackdropTemplate")
  background:SetWidth(self.bar:GetWidth())
  background:SetHeight(self.bar:GetHeight())
  background:SetPoint("CENTER", 0, 0)
  background:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", tile = true, tileSize = 32, insets = {left = 1, right = 1, top = 1, bottom = 1}})
  background:SetBackdropColor(0, 0, 0, 0)
  background:SetFrameLevel(self.frame:GetFrameLevel() + 1)

  self.progress = CreateFrame("Frame", "$parentProgress", self.bar, BackdropTemplateMixin and "BackdropTemplate")
  self.progress:SetWidth(5)
  self.progress:SetHeight(self.bar:GetHeight())
  self.progress:SetPoint("LEFT", 0, 0)
  self.progress:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", tile = true, tileSize = 32, insets = {left = 1, right = 1, top = 1, bottom = 1}})
  self.progress:SetBackdropColor(0, 1, 0, 0.5)
  self.progress:SetFrameLevel(self.frame:GetFrameLevel() + 1)
end

function OrganizedPanicProgressBar:CreateLabelUI()
  self.label = self.bar:CreateFontString("FontString", "OVERLAY")
  self.label:SetFont("Fonts\\FRIZQT__.TTF", 10, "MONOCHROME")
  self.label:SetPoint("RIGHT", self.bar, 0, 0)
  self.label:SetTextColor(1.0, 1.0, 1.0, 0.50)
end

function OrganizedPanicProgressBar:UpdateUI()
  local percent = self.value / self.max
  if percent > 1 then percent = 1 end
  local width = math.floor(self.frame:GetWidth() * percent)
  self.progress:SetWidth(width)
  self.label:SetText(self.text)
end

function OrganizedPanicProgressBar:GetFrame()
  return self.frame
end

function OrganizedPanicProgressBar:Set(text, value, max)
  self.text = text
  self.value = value
  self.max = max
  self:UpdateUI()
end

function OrganizedPanicProgressBar:SetValue(value)
  self.value = value
  self:UpdateUI()
end

function OrganizedPanicProgressBar:SetMax(max)
  self.max = max
  self:UpdateUI()
end

function OrganizedPanicProgressBar:SetText(text)
  self.text = text
  self:UpdateUI()
end

function OrganizedPanicProgressBar:Show()
  self.progress:Show()
end

function OrganizedPanicProgressBar:Hide()
  self.progress:Hide()
end
