--
-- OrganizedPanicSpellCard
--
-- Copyright (c) 2021 Dana Basken
--

local _, NS = ...

local AceEvent = LibStub("AceEvent-3.0")

OrganizedPanicSpellCard = {}
OrganizedPanicSpellCard.__index = OrganizedPanicSpellCard

local FRAME_HEIGHT = 12

function OrganizedPanicSpellCard:Create(spell, member, parentFrame)
  local instance = {}
  setmetatable(instance, OrganizedPanicSpellCard)
  instance:Construct(spell, member, parentFrame)
  return instance
end

function OrganizedPanicSpellCard:Construct(spell, member, parentFrame)
  self.spell = spell
  self.member = member
  self.parentFrame = parentFrame
  self.frameBackground = {red = 0, green = 0, blue = 0, alpha = 0}
  self:CreateUI()
  self:RefreshUI()
  AceEvent.RegisterMessage(self, "CASTING_STATE_CHANGE")
  AceEvent.RegisterMessage(self, "SPELL_STATE_CHANGE")
  AceEvent.RegisterMessage(self, "SPELL_COOLDOWN")
  self:UpdateFrameBackground()
end

function OrganizedPanicSpellCard:CreateUI()
  self.frame = CreateFrame("Frame", "$parentSpellCard", self.parentFrame, BackdropTemplateMixin and "BackdropTemplate")
  self.frame:SetWidth(self.parentFrame:GetWidth())
  self.frame:SetHeight(FRAME_HEIGHT)
  self.frame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", tile = true, tileSize = 32})
  self:SetPosition(0, 0)
  self.frame.progress = OrganizedPanicProgressBar:Create(self.frame, self.spell.duration, self.spell.duration)
end

function OrganizedPanicSpellCard:RefreshUI()
  self.frame.progress:Set(("%s: %3.0fs"):format(self.spell.name, self.spell.remaining), self.spell.remaining, self.spell.duration)
end

function OrganizedPanicSpellCard:CASTING_STATE_CHANGE(event, name, state, spellId)
  if self.member.name == name and self.spell.id == spellId then
    if state == "SPELL_START" then
      self.frameBackground = {red = 0, green = 1, blue = 0, alpha = 1}
      self:UpdateFrameBackground()
    end
    if state == "SPELL_STOP" then
      self.frameBackground = {red = 0, green = 0, blue = 0, alpha = 0}
      self:UpdateFrameBackground()
    end
  end
end

function OrganizedPanicSpellCard:SPELL_STATE_CHANGE(event, name, spellId, state)
  if self.member.name == name and self.spell.id == spellId then
    if state == "COOLDOWN" then
      self.frame.progress:Show()
    end
    if state == "READY" then
      self.frame.progress:Hide()
    end
    self:RefreshUI()
  end
end

function OrganizedPanicSpellCard:SPELL_COOLDOWN(event, name, spellId, remaining)
  if self.member.name == name and self.spell.id == spellId then self:RefreshUI() end
end

function OrganizedPanicSpellCard:SetPosition(x, y)
  self.frame:SetPoint("TOPLEFT", self.parentFrame, x, -y)
end

function OrganizedPanicSpellCard:GetHeight()
  return self.frame:GetHeight()
end

function OrganizedPanicSpellCard:GetRank()
  return self.spell.remaining
end

function OrganizedPanicSpellCard:Remove()
  self.frame:Hide()
  self.frame:SetParent(nil)
  self.frame:ClearAllPoints()
end

function OrganizedPanicSpellCard:UpdateFrameBackground()
  local alpha = self.frameBackground.alpha
  if NS.db.profile.windowLocked and alpha > 0 then alpha = 0.5 end
  self.frame:SetBackdropColor(self.frameBackground.red, self.frameBackground.green, self.frameBackground.blue, alpha)
end

function OrganizedPanicSpellCard:SetLocked(locked)
  -- self:UpdateFrameBackground()
end
