--
-- StateMachine
--
-- Copyright (c) 2020-2021 Dana Basken
--

local AceEvent = LibStub("AceEvent-3.0")

StateMachine = {}
StateMachine.__index = StateMachine

function StateMachine:Create(id, initial)
  local instance = {}
  setmetatable(instance, StateMachine)
  instance:Construct(id, initial)
  return instance
end

function StateMachine:Construct(id, initial)
  self.id = id
  self.initial = initial
  self:Reset()
end

function StateMachine:Reset()
  self:SetState(self.initial)
end

function StateMachine:SetState(state)
  if state == nil then state = '' end
  if state ~= self.state then
    local lastState = self.state
    self.state = state
    self:OnStateChange(lastState)
  end
end

function StateMachine:OnStateChange(lastState)
  AceEvent:SendMessage(("STATE_CHANGE_%s"):format(self.id), self.state, lastState)
end
